#include <stdio.h>
#include <assert.h>
#include <unistd.h>
#include "maestro.h"
#include <stdint.h>

#include "ecmwf_config.h"

int main(int argc, char* argv[])
{
    mstro_cdo_selector selector = NULL;
    mstro_subscription subscription = NULL;

    assert(MSTRO_OK == mstro_init(getenv("MSTRO_WORKFLOW_NAME"), getenv("MSTRO_COMPONENT_NAME"), 0));

    assert(MSTRO_OK == mstro_cdo_selector_create(NULL, NULL, "(and (.maestro.ecmwf.step = 2) (.maestro.ecmwf.number = 1))", &selector));
    assert(MSTRO_OK == mstro_subscribe(selector, MSTRO_POOL_EVENT_OFFER, false, &subscription));
    assert(MSTRO_OK == mstro_cdo_selector_dispose(selector));

    mstro_pool_event e = NULL;
    assert(MSTRO_OK == mstro_subscription_wait(subscription, &e));
    if (e != NULL) {
        fprintf(stdout, "Offer event: %p, kind: %d, cdo_name: %s\n", (void*)e, e->kind, e->offer.cdo_name);
        mstro_cdo cdo = NULL;
        assert(MSTRO_OK == mstro_cdo_declare(e->offer.cdo_name, MSTRO_ATTR_DEFAULT, &cdo));
        assert(MSTRO_OK == mstro_cdo_require(cdo));
        assert(MSTRO_OK == mstro_cdo_demand(cdo));

        enum mstro_cdo_attr_value_type ttype;
        const int64_t* tval = NULL;
        assert(MSTRO_OK == mstro_cdo_attribute_get(cdo, ".maestro.ecmwf.step", &ttype, (const void**)&tval));
        assert(ttype == MSTRO_CDO_ATTR_VALUE_int64);
        fprintf(stdout, "Step = %" PRIi64 "\n", *tval);

        assert(MSTRO_OK == mstro_cdo_dispose(cdo));
    }

    assert(MSTRO_OK == mstro_finalize());

    return 0;
}
